@@ -1,10 +1,8 @@
-// -*- C++ -*-
-
 /* 
  * Gnome Chemistry Utils
  * element.cc 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,21 +22,20 @@
 
 #include "config.h"
 #include "element.h"
-#include "xml-utils.h"
 #include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <libxml/xmlmemory.h>
 #include <glib.h>
+#include <locale.h>
 #include <vector>
 #include <map>
 #include <set>
 #include <string>
 #include <libintl.h>
+#include <math.h>
+#include <string.h>
 #include <glib/gi18n-lib.h>
 #include <goffice/utils/go-math.h>
-#include <clocale>
-#include <cmath>
-#include <cstring>
 
 static set<string>units;
 
@@ -51,15 +48,6 @@ static void ReadValue (char const *source, GcuValue &value)
 	value.delta = (*buf == '(')? strtol (buf + 1, NULL, 10): 0;
 }
 
-static void ReadDimensionalValue (char const *source, GcuDimensionalValue &value)
-{
-	char *buf, *dot;
-	value.value = strtod (source, &buf);
-	dot = strchr (source, '.');
-	value.prec = (dot)? buf - dot - 1: 0;
-	value.delta = (*buf == '(')? strtol (buf + 1, NULL, 10): 0;
-}
-
 namespace gcu
 {
 
@@ -87,7 +75,7 @@ EltTable Table;
 
 EltTable::EltTable()
 {
-	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+	bindtextdomain (GETTEXT_PACKAGE, DATADIR"/locale");
 #ifdef ENABLE_NLS
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
@@ -103,21 +91,21 @@ EltTable::EltTable()
 	Langs["it"] = _("Italian");
 	Langs["pl"] = _("Polish");
 	Langs["ru"] = _("Russian");
-	if (!(xml = xmlParseFile (PKGDATADIR"/elements.xml")))
-		g_error (g_strconcat (_("Can't find and read:"), " ", "elements.xml", NULL));
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/elements.xml")))
+	{
+		g_error (_("Can't find and read elements.xml"));
+	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata"))
-		g_error (g_strconcat( _("Incorrect file format:"), " ", "elements.xml", NULL));
+	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: elements.xml"));
 	node = node->children;
 	Element* Elt;
 	while (node)
 	{
 		if (strcmp ((const char*) node->name, "text"))
 		{
-			if (strcmp ((const char*) node->name, "element"))
-				g_error (g_strconcat( _("Incorrect file format:"), " ", "elements.xml", NULL));
+			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: elements.xml"));
 			buf = (char*) xmlGetProp (node, (xmlChar*) "symbol");
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = new Element (Z = atoi (num), buf);
@@ -308,11 +296,19 @@ Element::Element(int Z, const char* Symbol)
 Element::~Element()
 {
 	while (!m_radii.empty()) {
-		delete m_radii.back();
+		GcuAtomicRadius *radius = m_radii.back();
+		if (radius) {
+			if (radius->scale) g_free(radius->scale);
+			delete radius;
+		}
 		m_radii.pop_back();
 	}
 	while (!m_en.empty()) {
-		delete m_en.back();
+		GcuElectronegativity* en = m_en.back();
+		if (en) {
+			if (en->scale) g_free(en->scale);
+			delete en;
+		}
 		m_en.pop_back();
 	}
 	while (!m_isotopes.empty ()) {
@@ -423,13 +419,14 @@ void Element::LoadRadii ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (PKGDATADIR"/radii.xml")))
-		g_error (g_strconcat (_("Can't find and read:"), " ", "radii.xml", NULL));
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/radii.xml")))
+	{
+		g_error (_("Can't find and read radii.xml"));
+	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata"))
-		g_error (g_strconcat( _("Incorrect file format:"), " ", "radii.xml", NULL));
+	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: radii.xml"));
 	node = node->children;
 	Element* Elt;
 	set<string>::iterator it = units.find ("pm");
@@ -440,8 +437,7 @@ void Element::LoadRadii ()
 	while (node) {
 		if (strcmp ((const char*) node->name, "text"))
 		{
-			if (strcmp ((const char*) node->name, "element"))
-				g_error (g_strconcat (_("Incorrect file format:"), " ", "radii.xml", NULL));
+			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: radii.xml"));
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = Table[Z = atoi (num)];
 			child = node->children;
@@ -469,7 +465,7 @@ void Element::LoadRadii ()
 					}
 					buf = (char*) xmlGetProp (child, (xmlChar*) "scale");
 					if (buf) {
-						radius->scale = GetStaticScale (buf);
+						radius->scale = g_strdup (buf);
 						xmlFree (buf);
 					} else
 						radius->scale = NULL;
@@ -487,14 +483,14 @@ void Element::LoadRadii ()
 						radius->cn = -1;
 					buf = (char*) xmlGetProp (child, (xmlChar*)"spin");
 					if ((!buf) ||
-						((!((!strcmp (buf, "low")) && (radius->spin = GCU_LOW_SPIN))) &&
-						(!((!strcmp (buf, "high")) && (radius->spin = GCU_HIGH_SPIN)))))
+						(!((!strcmp (buf, "low")) && (radius->spin = GCU_LOW_SPIN))) &&
+						(!((!strcmp (buf, "high")) && (radius->spin = GCU_HIGH_SPIN))))
 						radius->spin = GCU_N_A_SPIN;
 					if (buf)
 						xmlFree (buf);
 					buf = (char*) xmlGetProp (child, (xmlChar*) "value");
 					if (buf) {
-						ReadDimensionalValue (buf, radius->value) ;
+						ReadValue (buf, (GcuValue&) radius->value) ;
 						radius->value.unit = (*it).c_str ();
 						Elt->m_radii.push_back (radius);
 						xmlFree (buf);
@@ -523,19 +519,19 @@ void Element::LoadElectronicProps ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (PKGDATADIR"/elecprops.xml")))
-		g_error (g_strconcat (_("Can't find and read:"), " ", "elecprops.xml", NULL));
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/elecprops.xml")))
+	{
+		g_error (_("Can't find and read elecprops.xml"));
+	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata"))
-		g_error (g_strconcat( _("Incorrect file format:"), " ", "elecprops.xml", NULL));
+	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: elecprops.xml"));
 	node = node->children;
 	Element* Elt;
 	while (node) {
 		if (strcmp ((const char*) node->name, "text")) {
-			if (strcmp ((const char*) node->name, "element"))
-				g_error (g_strconcat( _("Incorrect file format:"), " ", "elecprops.xml", NULL));
+			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: elecprops.xml"));
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = Table[Z = atoi (num)];
 			child = node->children;
@@ -549,7 +545,7 @@ void Element::LoadElectronicProps ()
 					en->Z = Z;	//FIXME: is it really useful there?
 					buf = (char*) xmlGetProp (child, (xmlChar*) "scale");
 					if (buf) {
-						en->scale = GetStaticScale (buf);
+						en->scale = g_strdup (buf);
 						xmlFree (buf);
 					} else
 						en->scale = NULL;
@@ -681,21 +677,21 @@ void Element::LoadIsotopes ()
 	static bool loaded = false;
 	if (loaded)
 		return;
-	if (!(xml = xmlParseFile (PKGDATADIR"/isotopes.xml")))
-		g_error (g_strconcat (_("Can't find and read:"), " ", "isotopes.xml", NULL));
+	if (!(xml = xmlParseFile (DATADIR"/gchemutils-unstable/isotopes.xml")))
+	{
+		g_error (_("Can't find and read isotopes.xml"));
+	}
 	old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
 	xmlNode* node = xml->children, *child;
-	if (strcmp ((const char*) node->name, "gpdata"))
-		g_error (g_strconcat (_("Incorrect file format:"), " ", "isotopes.xml", NULL));
+	if (strcmp ((const char*) node->name, "gpdata")) g_error (_("Incorrect file format: isotopes.xml"));
 	node = node->children;
 	Element *Elt;
 	Isotope *Is;
 	int minA, maxA, niso;
 	while (node) {
 		if (strcmp ((const char*) node->name, "text")) {
-			if (strcmp ((const char*) node->name, "element"))
-				g_error (g_strconcat (_("Incorrect file format:"), " ", "isotopes.xml", NULL));
+			if (strcmp ((const char*) node->name, "element")) g_error (_("Incorrect file format: isotopes.xml"));
 			minA = maxA = niso = 0;
 			num = (char*) xmlGetProp (node, (xmlChar*) "Z");
 			Elt = Table[Z = atoi (num)];
